@@ -6,6 +6,7 @@ import os
 import random
 import sys
 import time
+import braceexpand
 from dataclasses import dataclass
 from multiprocessing import Value
 
@@ -71,8 +72,29 @@ class DataInfo:
             self.sampler.set_epoch(epoch)
 
 
+def expand_urls(urls, weights=None):
+    if weights is None:
+        expanded_urls = wds.shardlists.expand_urls(urls)
+        return expanded_urls, None
+    if isinstance(urls, str):
+        urllist = urls.split("::")
+        weights = weights.split('::')
+        assert len(weights) == len(urllist), f"Expected the number of data components ({len(urllist)}) and weights({len(weights)}) to match."
+        weights = [float(weight) for weight in weights]
+        all_urls, all_weights = [], []
+        for url, weight in zip(urllist, weights):
+            expanded_url = list(braceexpand.braceexpand(url))
+            expanded_weights = [weight for _ in expanded_url]
+            all_urls.extend(expanded_url)
+            all_weights.extend(expanded_weights)
+        return all_urls, all_weights
+    else:
+        all_urls = list(urls)
+        return all_urls, weights
+
+
 def get_dataset_size(shards):
-    shards_list = wds.shardlists.expand_urls(shards)
+    shards_list, _ = expand_urls(shards)
     dir_path = os.path.dirname(shards_list[0])
     sizes_filename = os.path.join(dir_path, 'sizes.json')
     len_filename = os.path.join(dir_path, '__len__')
@@ -255,6 +277,7 @@ class ResampledShards2(IterableDataset):
     def __init__(
         self,
         urls,
+        weights=None,
         nshards=sys.maxsize,
         worker_seed=None,
         deterministic=False,
@@ -265,8 +288,11 @@ class ResampledShards2(IterableDataset):
         :param urls: a list of URLs as a Python list or brace notation string
         """
         super().__init__()
-        urls = wds.shardlists.expand_urls(urls)
+        urls, weights = expand_urls(urls, weights)
         self.urls = urls
+        self.weights = weights
+        if self.weights is not None:
+            assert len(self.urls) == len(self.weights), f"Number of urls {len(self.urls)} and weights {len(self.weights)} should match."
         assert isinstance(self.urls[0], str)
         self.nshards = nshards
         self.rng = random.Random()
@@ -292,28 +318,10 @@ class ResampledShards2(IterableDataset):
                 seed = self.worker_seed() + epoch
             self.rng.seed(seed)
         for _ in range(self.nshards):
-            yield dict(url=self.rng.choice(self.urls))
-
-
-class WeightedSampler(IterableDataset):
-    def __init__(self, datasets, weights=None):
-        super().__init__()
-        self.datasets = datasets
-        self.rng = random.Random()
-        if weights is None:
-            weights = [1 for _ in self.datasets]
-        self.weights = weights
-    
-    def __iter__(self):
-        sources = [iter(ds) for ds in self.datasets]
-        idxs = list(range(len(sources)))
-        for _ in range(sys.maxsize):
-            idx = self.rng.choices(idxs, weights=self.weights, k=1)[0]
-            try:
-                sample = next(sources[idx])
-                yield sample
-            except StopIteration:
-                return
+            if self.weights is None:
+                yield dict(url=self.rng.choice(self.urls))
+            else:
+                yield dict(url=self.rng.choices(self.urls, weights=self.weights, k=1)[0])
 
 
 def get_wds_dataset(args, preprocess_img, is_train, epoch=0, floor=False, tokenizer=None):
@@ -334,62 +342,47 @@ def get_wds_dataset(args, preprocess_img, is_train, epoch=0, floor=False, tokeni
 
     shared_epoch = SharedEpoch(epoch=epoch)  # create a shared epoch store to sync epoch to dataloader worker proc
     
-    def build_pipeline(input_shards):
-        if resampled:
-            pipeline = [ResampledShards2(input_shards, deterministic=True, epoch=shared_epoch)]
-        else:
-            pipeline = [wds.SimpleShardList(input_shards)]
+    if resampled:
+        pipeline = [ResampledShards2(input_shards, weights=args.train_data_upsampling_factors, deterministic=True, epoch=shared_epoch)]
+    else:
+        assert args.train_data_upsampling_factors is None, "--train_data_upsampling_factors is only supported when sampling with replacement (together with --dataset-resampled)."
+        pipeline = [wds.SimpleShardList(input_shards)]
 
-        # at this point we have an iterator over all the shards
-        if is_train:
-            if not resampled:
-                pipeline.extend([
-                    detshuffle2(
-                        bufsize=_SHARD_SHUFFLE_SIZE,
-                        initial=_SHARD_SHUFFLE_INITIAL,
-                        seed=args.seed,
-                        epoch=shared_epoch,
-                    ),
-                    wds.split_by_node,
-                    wds.split_by_worker,
-                ])
+    # at this point we have an iterator over all the shards
+    if is_train:
+        if not resampled:
             pipeline.extend([
-                # at this point, we have an iterator over the shards assigned to each worker at each node
-                tarfile_to_samples_nothrow,  # wds.tarfile_to_samples(handler=log_and_continue),
-                wds.shuffle(
-                    bufsize=_SAMPLE_SHUFFLE_SIZE,
-                    initial=_SAMPLE_SHUFFLE_INITIAL,
+                detshuffle2(
+                    bufsize=_SHARD_SHUFFLE_SIZE,
+                    initial=_SHARD_SHUFFLE_INITIAL,
+                    seed=args.seed,
+                    epoch=shared_epoch,
                 ),
-            ])
-        else:
-            pipeline.extend([
+                wds.split_by_node,
                 wds.split_by_worker,
-                # at this point, we have an iterator over the shards assigned to each worker
-                wds.tarfile_to_samples(handler=log_and_continue),
             ])
         pipeline.extend([
-            wds.select(filter_no_caption_or_no_image),
-            wds.decode("pilrgb", handler=log_and_continue),
-            wds.rename(image="jpg;png;jpeg;webp", text="txt"),
-            wds.map_dict(image=preprocess_img, text=lambda text: tokenizer(text)[0]),
-            wds.to_tuple("image", "text"),
+            # at this point, we have an iterator over the shards assigned to each worker at each node
+            tarfile_to_samples_nothrow,  # wds.tarfile_to_samples(handler=log_and_continue),
+            wds.shuffle(
+                bufsize=_SAMPLE_SHUFFLE_SIZE,
+                initial=_SAMPLE_SHUFFLE_INITIAL,
+            ),
         ])
-
-        return pipeline
-    
-    if args.train_data_weights is not None:
-        weights = args.train_data_weights
-        weights = [float(w) for w in weights.split('::')]
-        input_shard_list = input_shards.split('::')
-        assert len(input_shard_list) == len(weights), f"Expected the number of data components ({len(input_shard_list)}) and weights ({len(weights)}) to match."
-        datasets = [wds.DataPipeline(*build_pipeline(shards)) for shards in input_shard_list]
-        pipeline = [WeightedSampler(datasets, weights=weights)]
     else:
-        pipeline = build_pipeline(input_shards)
-    
-    pipeline.append(
+        pipeline.extend([
+            wds.split_by_worker,
+            # at this point, we have an iterator over the shards assigned to each worker
+            wds.tarfile_to_samples(handler=log_and_continue),
+        ])
+    pipeline.extend([
+        wds.select(filter_no_caption_or_no_image),
+        wds.decode("pilrgb", handler=log_and_continue),
+        wds.rename(image="jpg;png;jpeg;webp", text="txt"),
+        wds.map_dict(image=preprocess_img, text=lambda text: tokenizer(text)[0]),
+        wds.to_tuple("image", "text"),
         wds.batched(args.batch_size, partial=not is_train)
-    )
+    ])
 
     dataset = wds.DataPipeline(*pipeline)
 
